-- |Print all source packages that build depend on the given binary package.
module Main where

import Control.Monad.Trans (lift, liftIO)
import Data.List (intercalate, nub, sort)
import Data.Maybe (fromJust)
import Debian.Control
import Debian.Relation.ByteString
import Debian.Repo.Cache (buildArchOfRoot)
import Debian.Repo.IO (AptIOT, runAptIO)
import Debian.Repo.Slice (verifySourcesList)
import Debian.Repo.Types (EnvRoot(..), ReleaseName(..), NamedSliceList(..), SliceList(..), SliceName(..),
                          AptImage(..), AptCache(..), SourcePackage(..), PackageID(..))
import Debian.Repo.AptImage (updateAptEnv)
import Extra.CIO
import Extra.TIO
import Prelude hiding (putStrLn)

import Debian.Repo.SourcesList (parseSourcesList)

main :: IO ()
main = runTIO (setVerbosity 3 defStyle) (runAptIO main')

main' :: AptIOT TIO ()
main' =
    do os <- findEnv "/tmp/test" (EnvRoot root)
       case os of
         Nothing -> error "No environment found"
         Just os ->
             do os' <- updateAptEnv os
                let packages = aptImageSourcePackages os'
                let deps = filter (hasGHCDependency . sourceParagraph) packages
                liftIO (putStrLn ("Packages that build-depend on ghc6:\n  " ++
                       		  intercalate "\n  " (nub (sort (map (format . sourcePackageID) deps)))))
    where
      format x = packageName x	-- ++ "-" ++ show (packageVersion x)
      hasGHCDependency para = 
          case lookupP "Build-Depends" para of
            Just (Field (_, s)) ->
                case parseRelations s of
                  Right xs -> any (any isGHCRel) xs
                  Left e -> False	-- ePutStrLn ("Invalid dependency list: " ++ s)
            Nothing -> False
      isGHCRel (Rel "ghc6" _ _) = True
      isGHCRel _ = False
      root = "/home/david/.autobuilder/dists/hardy-seereason/clean-Moderate"

findEnv :: CIO m
        => FilePath
        -> EnvRoot
        -> AptIOT m (Maybe AptImage)
findEnv cacheDir root =
    do arch <- liftIO buildArchOfRoot
       sources <- liftIO (readFile (rootPath root ++ "/etc/apt/sources.list") >>= return . parseSourcesList)
       distro <- verifySourcesList (Just root) sources
       return . Just $ AptImage { aptGlobalCacheDir = cacheDir
                                , aptImageRoot = root
                                , aptImageArch = arch
                                , aptImageSliceList = distro
                                , aptImageReleaseName = ReleaseName {relName = "test"}
                                , aptImageSourcePackages = []
                                , aptImageBinaryPackages = [] }
    where
      sourcesPath = rootPath root ++ "/etc/apt/sources.list"
